import numpy as np
import hepfile as hep


def sep_cols_data_ids(filename):

    #with open(filename) as input:
    #    cols = input.read().split('\n', 1)[0].split(",")[1:]

    #data = np.loadtxt(filename, unpack=True, dtype=str,
    #                  delimiter=",", skiprows=1)
    #data_ID = data[0].astype(np.int32)
    #data = data[1:]
    data = np.loadtxt(filename, unpack=True, dtype=str,
                     delimiter=",", comments = '$')

    data_ID = data[0][1:].astype(np.int32)
    cols = data[1:, 0].tolist()
    data = data[1:, 1:]
    return cols, data, data_ID


def setup_group(data, group_name, counter, datasets):
    hep.create_group(data, group_name=group_name, counter=counter)
    #for dset in datasets:
    hep.create_dataset(data, datasets, group=group_name, dtype = str)


def fill_event_group(event, i, group_name, cols, counters, data):
    for j in range(len(counters)):
        if counters[j] == i:
            event[f'{group_name}/ID'] += 1
            for k in range(len(cols)):
                event[f'{group_name}/{cols[k]}'].append(data[k][j])


p_cols, p_data, p_ID = sep_cols_data_ids('sheet1.csv')
v_cols, v_data, v_ID = sep_cols_data_ids('sheet2.csv')
h_cols, h_data, h_ID = sep_cols_data_ids('sheet3.csv')

min_ID = min(min(p_ID), min(v_ID), min(h_ID))
max_ID = max(max(p_ID), max(v_ID), max(h_ID))

town = hep.initialize()
setup_group(town, 'people', 'ID', p_cols)
setup_group(town, 'vehicles', 'ID', v_cols)
setup_group(town, 'houses', 'ID', h_cols)
bucket = hep.create_single_bucket(town)



for i in range(min_ID, max_ID+1):
    fill_event_group(bucket, i, 'people', p_cols, p_ID, p_data)
    fill_event_group(bucket, i, 'vehicles', v_cols, v_ID, v_data)
    fill_event_group(bucket, i, 'houses', h_cols, h_ID, h_data)
    hep.pack(town, bucket)

hep.write_to_file("town_hep.hdf5", town, force_single_precision=False)



import numpy as np
import sys
#import hepfile

# For development
sys.path.append('../src/hepfile')
import write as hepfile

data = hepfile.initialize()

hepfile.create_group(data,'jet',counter='njet')
hepfile.create_dataset(data,['e','px','py','pz'],group='jet',dtype=float)
hepfile.create_dataset(data,['algorithm'],group='jet',dtype=int)
hepfile.create_dataset(data,['words'],group='jet',dtype=str)

hepfile.create_group(data,'muons',counter='nmuon')
hepfile.create_dataset(data,['e','px','py','pz'],group='muons',dtype=float)

hepfile.create_dataset(data,['METpx','METpy'],dtype=float)

event = hepfile.create_single_bucket(data)

rando_words = ["hi", "bye", "ciao", "aloha"]

#'''
for i in range(0,10000):

    #hepfile.clear_event(event)

    njet = 17
    event['jet/njet'] = njet

    for n in range(njet):
        event['jet/e'].append(np.random.random())
        event['jet/px'].append(np.random.random())
        event['jet/py'].append(np.random.random())
        event['jet/pz'].append(np.random.random())

        event['jet/algorithm'].append(np.random.randint(-1,1))

        event['jet/words'].append(np.random.choice(rando_words))

    event['METpx'] = np.random.random()
    event['METpy'] = np.random.random()

    #hepfile.pack(data,event,EMPTY_OUT_BUCKET=False)
    return_value = hepfile.pack(data,event,STRICT_CHECKING=True)
    if return_value != 0:
        exit()

print("Writing the file...")
#hdfile = hepfile.write_to_file('output.hdf5',data)
hdfile = hepfile.write_to_file('output.hdf5',data,comp_type='gzip',comp_opts=9,verbose=True)
#'''




import numpy as np
import h5py as h5
import datetime
import sys
import hepfile


################################################################################
def initialize():
    """ Creates an empty data dictionary

    Returns:

        **data** (dict): An empty data dictionary

    """

    data = {}
    data["_GROUPS_"] = {}
    data["_MAP_DATASETS_TO_COUNTERS_"] = {}
    data["_LIST_OF_COUNTERS_"] = []

    # For singleton entries, variables with only one entry per bucket.
    data["_GROUPS_"]["_SINGLETONS_GROUP_"] = ["COUNTER"]
    data["_MAP_DATASETS_TO_COUNTERS_"]["_SINGLETONS_GROUP_"] = "_SINGLETONS_GROUP_/COUNTER"
    data["_LIST_OF_COUNTERS_"].append("_SINGLETONS_GROUP_/COUNTER")

    data["_SINGLETONS_GROUP_/COUNTER"] = []
    data["_MAP_DATASETS_TO_DATA_TYPES_"] = {}
    data["_MAP_DATASETS_TO_DATA_TYPES_"]["_SINGLETONS_GROUP_/COUNTER"] = int

    data["_PROTECTED_NAMES_"] = ["_PROTCTED_NAMES_",
                                 "_GROUPS_",
                                 "_MAP_DATASETS_TO_COUNTERS_",
                                 "_MAP_DATASETS_TO_DATA_TYPES_"
                                 "_LIST_OF_COUNTERS_",
                                 "_SINGLETONS_GROUP_/COUNTER",
                                 ]

    return data


################################################################################
def clear_bucket(bucket):
    """ Clears the data from the bucket dictionary - should the name of the function change?

    Args:
        **bucket** (dict): The dictionary to be cleared. This is designed to clear the data from
                      the lists in the **bucket** dictionary, but theoretically, it would
                      clear out the lists from any dictionary. 

    """

    for key in bucket.keys():

        if key == '_LIST_OF_COUNTERS_':
            continue

        if type(bucket[key]) == list:
            bucket[key].clear()
        elif type(bucket[key]) == int:
            if key in bucket['_LIST_OF_COUNTERS_']:
                bucket[key] = 0
            else:
                bucket[key] = -999
        elif type(bucket[key]) == float:
            bucket[key] = -999.0
        elif type(bucket[key]) == str:
            bucket[key] = "-999"


################################################################################
# Create a single bucket (dictionary) that will eventually be used to fill
# the overall dataset
################################################################################
def create_single_bucket(data):
    """ Creates an bucket dictionary that will be used to collect data and then
    packed into the the master data dictionary.

    Args:
        **data** (dict): Data dictionary that will hold all the data from the bucket.

    Returns:
        **bucket** (dict): The new bucket dictionary with keys and no bucket information

    """

    bucket = {}

    for k in data.keys():
        if k in data["_LIST_OF_COUNTERS_"]:
            bucket[k] = 0
        else:
            bucket[k] = data[k].copy()

    return bucket


################################################################################
# This adds a group in the dictionary, similar to
# a la CreateBranch in ROOT
################################################################################
def create_group(data, group_name, counter=None):
    """ Adds a group in the dictionary

    Args:
        **data** (dict): Dictionary to which the group will be added

        **group_name** (string): Name of the group to be added

        **counter** (string): Name of the counter key. None by default

    """

    # Check for slashes in the group name. We can't have them.
    if group_name.find('/')>=0:
        new_group_name = group_name.replace('/','-')
        print("----------------------------------------------------")
        print(f"Slashes / are not allowed in group names")
        print(f"Replacing / with - in group name {group_name}")
        print(f"The new name will be {new_group_name}")
        print("----------------------------------------------------")
        group_name = new_group_name 

    # Change name of variable, just to keep code more understandable
    counter_name = counter

    # Create a counter_name if the user has not specified one
    if counter_name is None:
        print("----------------------------------------------------")
        print(
            f"There is no counter to go with group \033[1m{group_name}\033[0m")
        print("Are you sure that's what you want?")
        counter_name = f"N_{group_name}"
        print(f"Creating a counter called \033[1m{counter_name}\033[0m")
        print("-----------------------------------------------------")

    # Check for slashes in the counter name. We can't have them.
    if counter_name.find('/')>=0:
        new_counter_name = counter_name.replace('/','-')
        print("----------------------------------------------------")
        print(f"Slashes / are not allowed in counter names")
        print(f"Replacing / with - in counter name {counter_name}")
        print(f"The new name will be {new_counter_name}")
        print("----------------------------------------------------")
        counter_name = new_counter_name 

    keys = data.keys()

    # Then put the group and any datasets in there next.
    keyfound = False
    for k in keys:
        if group_name == k:
            print(f"\033[1m{group_name}\033[0m is already in the dictionary!")
            keyfound = True
            break

    if keyfound == False:

        data["_GROUPS_"][group_name] = []
        print(f"Adding group \033[1m{group_name}\033[0m")

        data["_GROUPS_"][group_name].append(counter_name)
        full_counter_name = f"{group_name}/{counter_name}"

        data["_MAP_DATASETS_TO_COUNTERS_"][group_name] = full_counter_name
        data["_MAP_DATASETS_TO_DATA_TYPES_"][full_counter_name] = int

        if full_counter_name not in data["_LIST_OF_COUNTERS_"]:
            data["_LIST_OF_COUNTERS_"].append(full_counter_name)

        data[full_counter_name] = []
        print(f"Adding a counter for \033[1m{group_name}\033[0m as \033[1m{counter_name}\033[0m")

    return 0

################################################################################

################################################################################
# This adds a dataset to the dictionary, similar to
# a la CreateBranch in ROOT
#
# This can also add a dataset that is not associate with a group
################################################################################
def create_dataset(data, datasets, group=None, dtype=float):
    """ Adds a dataset to a group in a dictionary. If the group does not exist, it will be created.

    Args:
        **data** (dict): Dictionary that contains the group

        **datasets** (list): Dataset to be added to the group (This doesn't have to be a list)

        **group** (string): Name of group the dataset will be added to.  None by default

        **dtype** (type): The data type. None by default - I don't think this is every used 

    Returns:
        **-1**: If the group is None


    """

    if type(datasets) != list:
        datasets = [datasets]

    # Check for slashes in the group name. We can't have them.
    for i in range(len(datasets)):
        tempname = datasets[i]
        if tempname.find('/')>=0:
            new_dataset_name = tempname.replace('/','-')
            print("----------------------------------------------------")
            print(f"Slashes / are not allowed in dataset names")
            print(f"Replacing / with - in dataset name {tempname}")
            print(f"The new name will be {new_dataset_name}")
            print("----------------------------------------------------")
            datasets[i] = new_dataset_name 

    keys = data.keys()

    # These will be entries for the SINGLETON_GROUP, if there is no group passed in
    if group is None:

        for dataset in datasets:
            keyfound = False
            for k in data["_GROUPS_"]["_SINGLETONS_GROUP_"]:
                if dataset == k:
                    print(f"\033[1m{dataset}\033[0m is already in the dictionary!")
                    keyfound = True
            if keyfound == False:
                print(f"Adding dataset \033[1m{dataset}\033[0m to the dictionary as a SINGLETON.")
                data["_GROUPS_"]["_SINGLETONS_GROUP_"].append(dataset)
                data[dataset] = []
                data["_MAP_DATASETS_TO_COUNTERS_"][dataset] = "_SINGLETONS_GROUP_/COUNTER"

                data["_MAP_DATASETS_TO_DATA_TYPES_"][dataset] = dtype

        return 0

    # Put the counter in the dictionary first.
    keyfound = False
    for k in data["_GROUPS_"]:
        if group == k:
            keyfound = True

    # NEED TO FIX THIS PART SO THAT IT FINDS THE RIGHT COUNTER FROM THE GROUP
    if keyfound == False:
        print(f"Your group, \033[1m{group}\033[0m is not in the dictionary yet!")
        counter = f"N_{group}"
        print(f"Adding it, along with a counter of \033[1m{counter}\033[0m")
        create_group(data, group, counter=counter)

    # Then put the datasets into the group in there next.
    if type(datasets) != list:
        datasets = [datasets]

    for dataset in datasets:
        keyfound = False
        name = "%s/%s" % (group, dataset)
        for k in keys:
            if name == k:
                print(f"\033[1m{name}\033[0m is already in the dictionary!")
                keyfound = True
        if keyfound == False:
            print(f"Adding dataset \033[1m{dataset}\033[0m to the dictionary under group \033[1m{group}\033[0m.")
            data[name] = []
            data["_GROUPS_"][group].append(dataset)

            # Add a counter for this dataset for the group with which it is associated.
            counter = data["_MAP_DATASETS_TO_COUNTERS_"][group]
            # counter_name = "%s/%s" % (group,counter)
            data["_MAP_DATASETS_TO_COUNTERS_"][name] = counter

            data["_MAP_DATASETS_TO_DATA_TYPES_"][name] = dtype

    return 0


################################################################################
def pack(data, bucket, AUTO_SET_COUNTER=True, EMPTY_OUT_BUCKET=True, STRICT_CHECKING=False, verbose=False):
    """ Takes the data from an bucket and packs it into the data dictionary, 
    intelligently, so that it can be stored and extracted efficiently. 
    (This is analagous to the ROOT TTree::Fill() member function).

    Args:
        **data** (dict): Data dictionary to hold the entire dataset EDIT.

        **bucket** (dict): bucket to be packed into data.

        **EMPTY_OUT_BUCKET** (bool): If this is `True` then empty out the `bucket`
                                container in preparation for the next iteration. We used to ask the users to do
                                this "by hand" but now do it automatically by default. We allow the user to 
                                not do this, if they are running some sort of debugging. 

    """

    # Calculate the number of entries for each group and set the
    # value of that counter
    # This is all done in bucket
    if AUTO_SET_COUNTER:
        for group in data['_GROUPS_']:

            if verbose:
                print(f"group: {group}")

            datasets = data['_GROUPS_'][group]
            counter = data['_MAP_DATASETS_TO_COUNTERS_'][group]
            if counter == '_SINGLETONS_GROUP_/COUNTER':
                continue

            # Here we will calculate the values for the counters, based
            # on the size of the datasets
            counter_value = None

            # Loop over the datasets
            for d in datasets:
                full_dataset_name = group + "/" + d
                # Skip any counters
                if counter == full_dataset_name:
                    continue
                else:
                    # Grab the size of the first dataset
                    temp_counter_value = len(bucket[full_dataset_name])

                    # If we're not STRICT_CHECKING, then use that value for the
                    # counter and break the loop over the datasets, moving on
                    # to the next group.
                    if STRICT_CHECKING is False:
                        bucket[counter] = temp_counter_value
                        break
                    # Otherwise, we'll check that *all* the datasets have the same
                    # length.
                    else:
                        if counter_value is None:
                            counter_value = temp_counter_value
                            bucket[counter] = temp_counter_value
                        elif counter_value != temp_counter_value:
                            # In this case, we found two groups of different length!
                            # Print this to help the user identify their error
                            print(
                                f"Oh no!!!! Two datasets in group {group} have different sizes!")
                            for tempd in datasets:
                                temp_full_dataset_name = group + "/" + tempd
                                # Don't worry about the dataset
                                if counter == temp_full_dataset_name:
                                    continue
                                print(
                                    f"{tempd}: {len(bucket[temp_full_dataset_name])}")

                            # Return a value for the external program to catch.
                            return -1

    # Then pack the bucket into the data
    keys = list(bucket.keys())

    for key in keys:

        if (
            key == "_MAP_DATASETS_TO_COUNTERS_"
            or key == "_GROUPS_"
            or key == "_LIST_OF_COUNTERS_"
            or key == "_MAP_DATASETS_TO_DATA_TYPES_"
        ):
            continue

        # The singletons will only have 1 entry per bucket
        if key == "_SINGLETONS_GROUP_/COUNTER":
            data[key].append(1)
            continue

        if type(bucket[key]) == list:
            value = bucket[key]
            if len(value) > 0:
                data[key] += value
        else:
            # This is for counters and SINGLETONS
            if key in data["_GROUPS_"]["_SINGLETONS_GROUP_"]:
                if bucket[key] == None:
                    print(f"\n\033[1m{key}\033[0m is part of the SINGLETON group and is expected to have a value for each bucket.")
                    print("However it is None...exiting.\n")
                    exit()
                # Append the single value from the singletons
                else:
                    data[key].append(bucket[key])
            # Append the values to the counters
            else:
                data[key].append(bucket[key])

    # Clear out the bucket after it's been packed if that's what we want
    if EMPTY_OUT_BUCKET:
        clear_bucket(bucket)

    return 0

################################################################################


def convert_list_and_key_to_string_data(datalist, key):
    """ Converts data dictionary to a string

    Args:
        **datalist** (list): A list to be saved as a string.

    Returns:
        **key** (string): We will assume that this will be unpacked as a dictionary,
                      and this will be the key for the list in that dictionary.

    """

    a = np.string_(key)

    mydataset = []
    b = np.string_("")
    nvals = len(datalist)
    for i, val in enumerate(datalist):
        b += np.string_(val)
        if i < nvals - 1:
            b += np.string_("__:__")
    mydataset.append([a, b])

    return mydataset


################################################################################

################################################################################
def convert_dict_to_string_data(dictionary):
    """ Converts data dictionary to a string

    Args:
        **dictionary** (dict): Dictionary to be converted to a string

    Returns:
        **mydataset** (string): String representation of the dataset

    """

    keys = dictionary.keys()

    nkeys = len(keys)

    mydataset = []
    for i, key in enumerate(keys):
        a = np.string_(key)
        b = np.string_(dictionary[key])
        mydataset.append([a, b])

    return mydataset


################################################################################
# This function writes default attributes and metadata to a file.
################################################################################
def write_file_metadata(filename, mydict={}, write_default_values=True, append=True):
    """ Writes file metadata in the attributes of an HDF5 file

    Args:
    **filename** (string): Name of output file

    **mydict** (dictionary): Metadata desired by user

    **write_default_values** (boolean): True if user wants to write/update the 
                                        default metadata: date, hepfile version, 
                                        h5py version, numpy version, and Python 
                                        version, false if otherwise.

    **append** (boolean): True if user wants to keep older metadata, false otherwise.

    Returns:
    **hdoutfile** (HDF5): File with new metadata

    """

    hdoutfile = h5.File(filename, "a")

    non_metadata = ["_NUMBER_OF_BUCKETS_"]

    if not append:
        for key in hdoutfile.attr.keys():
            if key not in non_metadata:
                del hdoutfile.attrs[key]

    if write_default_values:
        hdoutfile.attrs["date"] = datetime.datetime.now().isoformat(sep=" ")
        hdoutfile.attrs["hepfile_version"] = hepfile.__version__
        hdoutfile.attrs["numpy_version"] = np.__version__
        hdoutfile.attrs["h5py_version"] = h5.__version__
        hdoutfile.attrs["python_version"] = sys.version

    for key in mydict:
        hdoutfile.attrs[key] = mydict[key]

    hdoutfile.close()
    print("Metadata added")
    return hdoutfile


################################################################################

################################################################################
def write_to_file(
    filename, data, comp_type=None, comp_opts=None, force_single_precision=True,  verbose=False
):
    """ Writes the selected data to an HDF5 file

    Args:
        **filename** (string): Name of output file

        **data** (dictionary): Data to be written into output file

        **comp_type** (string): Type of compression

        **force_single_precision** (boolean): True if data should be written in single precision

    Returns:
        **hdoutfile** (HDF5): File to which the data has been written 

    """

    hdoutfile = h5.File(filename, "w")

    _GROUPS_ = data["_GROUPS_"].keys()

    # Convert this to a 2xN array for writing to the hdf5 file.
    # This gives us one small list of informtion if we need to pull out
    # small chunks of data
    mydataset = convert_dict_to_string_data(data["_MAP_DATASETS_TO_COUNTERS_"])
    dset = hdoutfile.create_dataset(
        "_MAP_DATASETS_TO_COUNTERS_",
        data=mydataset,
        dtype="S256",
        compression=comp_type,
        compression_opts=comp_opts,
    )

    # Convert this to a 2xN array for writing to the hdf5 file.
    # This has the _GROUPS_ and the datasets in them.
    mydataset = convert_list_and_key_to_string_data(
        data["_GROUPS_"]["_SINGLETONS_GROUP_"], "_SINGLETONSGROUPFORSTORAGE_"
    )
    dset = hdoutfile.create_dataset(
        "_SINGLETONSGROUPFORSTORAGE_",
        data=mydataset,
        dtype="S256",
        compression=comp_type,
        compression_opts=comp_opts,
    )

    for group in _GROUPS_:

        hdoutfile.create_group(group)
        hdoutfile[group].attrs["counter"] = np.string_(
            data["_MAP_DATASETS_TO_COUNTERS_"][group]
        )

        datasets = data["_GROUPS_"][group]

        for dataset in datasets:

            name = None
            if group == "_SINGLETONS_GROUP_" and dataset is not "COUNTER":
                name = dataset
            else:
                name = "%s/%s" % (group, dataset)

            x = data[name]

            dataset_dtype = data['_MAP_DATASETS_TO_DATA_TYPES_'][name]

            if type(x) == list:
                x = np.array(x)

            # Do single precision only, unless specified
            if force_single_precision == True:
                if x.dtype == np.float64:
                    x = x.astype(np.float32)
                    dataset_dtype = np.float32

            if dataset_dtype is not str:
                hdoutfile.create_dataset(
                    name, data=x, compression=comp_type, compression_opts=comp_opts, dtype=dataset_dtype
                )
            else:
                # For writing strings, we need to make sure our strings are ascii and not Unicode
                #
                # See my question on StackOverflow and the super-helpful response!
                #
                # https://stackoverflow.com/questions/68500454/can-i-use-h5py-to-write-strings-to-an-hdf5-file-in-one-line-rather-than-looping
                dataset_dtype = h5.special_dtype(vlen=str)
                longest_word = len(max(x, key=len))
                arr = np.array(x, dtype='S'+str(longest_word))
                hdoutfile.create_dataset(
                    name, data=arr, dtype=dataset_dtype,  compression=comp_type, compression_opts=comp_opts)

            if (verbose):
                print(f"Writing to file {name} as type {str(dataset_dtype)}")

    # Get the number of buckets
    counters = data["_LIST_OF_COUNTERS_"]
    _NUMBER_OF_BUCKETS_ = -1
    prevcounter = None
    for i, countername in enumerate(counters):
        ncounter = len(data[countername])
        #print("%-32s has %-12d entries" % (countername, ncounter))
        print(f"{countername:<32s} has {ncounter:<12d} entries")
        if i > 0 and ncounter != _NUMBER_OF_BUCKETS_:
            print("-------- WARNING -----------")
            print(f"{countername} and {prevcounter} have differing numbers of entries!")
            print("-------- WARNING -----------")
            # SHOULD WE EXIT ON THIS?

        if _NUMBER_OF_BUCKETS_ < ncounter:
            _NUMBER_OF_BUCKETS_ = ncounter

        prevcounter = countername

    hdoutfile.attrs["_NUMBER_OF_BUCKETS_"] = _NUMBER_OF_BUCKETS_
    hdoutfile.close()

    write_file_metadata(filename)

    return hdoutfile



import awkward as ak
import hepfile.write 
import hepfile.read 
import numpy as np

################################################################################
#def unpack_awkward_arrays(data,keys):
def unpack_awkward_arrays(data,groups=None,datasets=None):

    #data,event = read.load(infilename)

    #if type(keys) is not list:
    #    keys = [keys]

    alldatasets = data['_LIST_OF_DATASETS_']
    allgroups = []
    for d in alldatasets:
        if d.find('/')>=0:
            allgroups.append(d.split('/')[0])

    allgroups = np.unique(allgroups)
    
    ak_arrays = {}

    for group in groups:
        print("group: ",group)
        ak_arrays[group] = {}
        #topkey = key.split('/')[0]
        #nkey = topkey + "/n"  + topkey
        for dataset in alldatasets:
            if dataset.find(group)>=0:

                print("dataset: ",dataset)

                if dataset in data['_LIST_OF_COUNTERS_'] or \
                   dataset == group:
                    continue

                nkey = data['_MAP_DATASETS_TO_COUNTERS_'][dataset]
                print('nkey: ',nkey)
                num = data[nkey]
                vals = data[dataset]
                print(vals)
                print(num)
                if type(vals[0]) is str:
                    vals = vals.astype(str)
                ak_array = ak.unflatten(vals,num)
                datasetname = dataset.split(group+'/')[-1]
                ak_arrays[group][datasetname] = ak_array

    return ak_arrays

################################################################################
'''
def pack_coffea_awkward_arrays(data,awkward_arrays,new_entries=True,names=None):

    if names is None:
        print("The names for the groups must be provided!")
        print("Nothing will be written!")
        return -1
    
    if type(awkward_arrays) is not list:
        awkward_arrays = [awkward_arrays]

    for arr,groupname in zip(awkward_arrays,names):
        write.create_group(data, groupname, counter="n"+groupname)
        for field in arr.fields:



    return 1
'''



import numpy as np
import awkward as ak
import uproot as uproot

import sys

#from coffea.nanoevents import NanoEventsFactory, NanoAODSchema

import hepfile as hp


infilename = sys.argv[1]

#print("Reading in {0}".format(infilename))

#events = NanoEventsFactory.from_root(infilename, schemaclass=NanoAODSchema).events()
#print(len(events))

#:wq

data,event = hp.load(infilename)
x = hp.unpack_awkward_arrays(data,['jet'])






import numpy as np
import matplotlib.pylab as plt
import time

#import hepfile

import sys

sys.path.append('../src/hepfile')
import read as hepfile


filename = sys.argv[1]

#data,event = hepfile.load(filename,subset=(0,100000))
#data,event = hepfile.load(filename,verbose=False)#,subset=10000)
#data,event = hepfile.load(filename,desired_datasets=['jet','muon'])
#data,event = hepfile.load(filename,desired_datasets=['jet'])
data,event = hepfile.load(filename,desired_datasets=['jet'],subset=(5,10))
#data,event = hepfile.load(filename,desired_datasets=['jet','muon'],subset=(0,100000))

if data is None:
    print("Exiting...")
    exit()

#print(data['list_of_counters'])

nbuckets_in_file = hepfile.get_nbuckets_in_file(filename)
print(f"nentries in file: {nbuckets_in_file}")

nbuckets = hepfile.get_nbuckets_in_data(data)
print(f"nentries: {nbuckets}")


energies = []

for i in range(0,nbuckets):

    if i%10000==0:
        print(i)

    hepfile.unpack(event,data,n=i)

    energy = event['jet/e']

    '''
    for e in energy:
        energies.append(e)
    '''
    energies += energy.tolist()


print(len(energies))

plt.figure()
plt.hist(energies,bins=100,range=(0,500))

#plt.show()



import numpy as np
import sys
#import hepfile

# For development
sys.path.append('../src/hepfile')
import write as hepfile

data = hepfile.initialize()

hepfile.create_group(data,'jet')#,counter='njet')




"""
A file description modeled after the ROOT analysis toolkit and common use-cases in High Energy Physics. Implemented in HDF5, hepfile stands for Heterogeneous Entries in Parallel-file. 
"""

__version__ = "0.1.3"

__all__ = ("__version__",)

from hepfile.read import *
from hepfile.write import *
from hepfile.awkward_tools import *



import numpy as np
import hepfile as hep

people = np.loadtxt('sheet1.csv', unpack=True, dtype=str,
                     delimiter=",")

#with open('sheet2.csv') as input:
#        cols = input.read().split('\n')
#        for col in cols:
#            print(len(col.split(',')))

vehicles = np.loadtxt('sheet2.csv', unpack=True, dtype=str,
                     delimiter=",", comments = '$')

houses = np.loadtxt('sheet3.csv', unpack=True, dtype=str,
                     delimiter=",", comments = '$')

people_ID = people[0][1:].astype(np.int32)
vehicles_ID = vehicles[0][1:].astype(np.int32)
houses_ID = houses[0][1:].astype(np.int32)

town = hep.initialize()

hep.create_group(town, 'people', counter ='ID')
hep.create_group(town, 'vehicles', counter='ID')
#hep.create_group(town, 'houses')

hep.create_dataset(town, ['First name','Last name','Gender ID',
                        'Highest degree-grade'], group = 'people', dtype = str)
hep.create_dataset(town, ['Age', 'Height', 'Yearly Income'], group = 'people', dtype = int)

hep.create_dataset(town, ['Type of vehicle','Gas-electric-human powered'], group = 'vehicles', dtype = str)
hep.create_dataset(town, ['# of riders', 'Year', 'Cost'], group = 'vehicles', dtype = int)

hep.create_dataset(town, ['House-apartment-condo'], dtype = str)
hep.create_dataset(town, ['# of bedrooms', 'Square footage','Year built',
                            'Estimate'], dtype = int)
hep.create_dataset(town, '# of bathrooms', dtype = float)

bucket = hep.create_single_bucket(town)

for i in range(0,4):
    for j in range(len(people_ID)):
        if people_ID[j] == i:
            bucket['people/First name'].append(people[1, j+1])
            bucket['people/Last name'].append(people[2, j+1])
            bucket['people/Gender ID'].append(people[3, j+1])
            bucket['people/Age'].append(people[4, j+1].astype(np.int32))
            bucket['people/Height'].append(people[5, j+1].astype(np.int32))
            bucket['people/Yearly Income'].append(people[6, j+1].astype(np.int32))
            bucket['people/Highest degree-grade'].append(people[6, j+1])
            bucket['people/ID'] += 1
    
    for j in range(len(vehicles_ID)):
        if vehicles_ID[j] == i:
            bucket['vehicles/Type of vehicle'].append(vehicles[1, j+1])
            bucket['vehicles/# of riders'].append(vehicles[2, j+1].astype(np.int32))
            bucket['vehicles/Gas-electric-human powered'].append(vehicles[3, j+1])
            bucket['vehicles/Year'].append(vehicles[4, j+1].astype(np.int32))
            bucket['vehicles/Cost'].append(vehicles[5, j+1].astype(np.int32))
            bucket['vehicles/ID'] += 1
    
    bucket['House-apartment-condo'] = houses[1, i+1]
    bucket['# of bedrooms'] = houses[2, i+1].astype(np.int32)
    bucket['# of bathrooms'] = houses[3, i+1].astype(np.float32)
    bucket['Square footage'] = houses[4, i+1].astype(np.int32)
    bucket['Year built'] = houses[5, i+1].astype(np.int32)
    bucket['Estimate'] = houses[6, i+1].astype(np.int32)
                        
    hep.pack(town, bucket)

hep.write_to_file('town_hep_long.hdf5', town,force_single_precision=False)




import numpy as np
import h5py as h5
import hepfile

import time

#import sys
# sys.path.append('./scripts')
#from write_h5hep_file_for_unit_tests import write_h5hep_file_for_unit_tests

from write_file_for_unit_tests import write_file_for_unit_tests


def isEmpty(dictionary):
    test = True
    print(dictionary.keys())
    for key in dictionary.keys():
        print(key)
        print(dictionary[key])
        print(type(dictionary[key]))
        if dictionary[key] is None:
            test = True
        elif type(dictionary[key]) == list or type(dictionary[key]) == np.ndarray:
            if len(dictionary[key]) > 0:
                test = False

    return test


def test_load():

    write_file_for_unit_tests()

    # This assumes you run nosetests from the h5hep directory and not
    # the tests directory.
    filename = "FOR_TESTS.hdf5"
    desired_datasets = ['jet', 'muon']
    subset = 5

    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, subset)

    assert isinstance(test_data, dict)
    assert isinstance(test_bucket, dict)

    assert isEmpty(test_bucket) == True
    assert isEmpty(test_data) == False

    # Testing desired_datasets
    assert "jet/e" in test_data.keys()
    assert "jet/e" in test_data.keys()
    assert "METpx" not in test_data.keys()
    assert "METpx" not in test_data.keys()

    # Testing subsets
    assert len(test_data["jet/njet"]) == 5

    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, 1000)

    assert len(test_data["jet/njet"]) == 10

    # Passing in a range of subsets
    subset = (0,4)
    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, subset=subset)
    assert len(test_data["jet/njet"]) == 4

    subset = (1,5)
    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, subset=subset)
    assert len(test_data["jet/njet"]) == 4

    subset = [1,5]
    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, subset=subset)
    assert len(test_data["jet/njet"]) == 4

    # Test for poor uses of subset
    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, [0,0])
    
    assert len(test_data['_LIST_OF_DATASETS_']) == 0
    assert len(test_bucket.keys()) == 0

    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, [10,0])
    
    assert len(test_data['_LIST_OF_DATASETS_']) == 0
    assert len(test_bucket.keys()) == 0

    test_data, test_bucket = hepfile.load(
        filename, False, desired_datasets, subset=0)
    
    assert len(test_data['_LIST_OF_DATASETS_']) == 0
    assert len(test_bucket.keys()) == 0

def test_unpack():

    # This assumes you run nosetests from the h5hep directory and not
    # the tests directory.
    filename = "FOR_TESTS.hdf5"
    desired_datasets = ['jet', 'muon']
    subset = 10

    bucket, data = hepfile.load(filename, False, desired_datasets, subset)

    hepfile.unpack(data, bucket)

    assert isEmpty(bucket) == False


def test_get_nbuckets_in_file():

    # This assumes you run nosetests from the h5hep directory and not
    # the tests directory.
    filename = "FOR_TESTS.hdf5"

    nbuckets = hepfile.get_nbuckets_in_file(filename)

    assert nbuckets == 10


def test_get_file_metadata():

    filename = "FOR_TESTS.hdf5"

    metadata = hepfile.get_file_metadata(filename)

    assert 'date' in metadata
    assert 'hepfile_version' in metadata
    assert 'h5py_version' in metadata
    assert 'numpy_version' in metadata
    assert 'python_version' in metadata

    # Check default attributes are strings
    assert isinstance(metadata['date'], str)
    assert isinstance(metadata['hepfile_version'], str)
    assert isinstance(metadata['h5py_version'], str)
    assert isinstance(metadata['numpy_version'], str)
    assert isinstance(metadata['python_version'], str)



import h5py as h5
import numpy as np

################################################################################
def load(filename=None, verbose=False, desired_datasets=None, subset=None):

    '''
    Reads all, or a subset of the data, from the HDF5 file to fill a data dictionary.
    Returns an empty dictionary to be filled later with data from individual buckets.

    Args:
	**filename** (string): Name of the input file
	
	**verbose** (boolean): True if debug output is required

	**desired_datasets** (list): Datasets to be read from input file, THIS IS REALLY
    STRING MATCHING SO THE USER COULD PASS IN A GROUP NAME. IS THIS RIGHT?

	**subset** (int): Number of buckets to be read from input file

    Returns:
	**data** (dict): Selected data from HDF5 file
	
	**bucket** (dict): An empty bucket dictionary to be filled by data from select buckets

    '''

    # Open the HDF5 file
    infile = None
    if filename != None:
        infile = h5.File(filename, "r+")
    else:
        print("No filename passed in! Can't open file.\n")
        return None,None

    # Create the initial data and bucket dictionary to hold the data
    data = {}
    bucket = {}

    # We'll fill the data dictionary with some extra fields, though we won't
    # need them all for the bucket
    data["_MAP_DATASETS_TO_COUNTERS_"] = {}
    data["_MAP_DATASETS_TO_INDEX_"] = {}
    data["_LIST_OF_COUNTERS_"] = []
    data["_LIST_OF_DATASETS_"] = []

    # Get the number of buckets.
    # In HEP (High Energy Physics), this would be the number of events
    data["_NUMBER_OF_BUCKETS_"] = infile.attrs["_NUMBER_OF_BUCKETS_"]

    # We might only read in a subset of the data though!
    if subset is not None:
        if type(subset) is tuple:
            subset = list(subset)

        if type(subset) is int:
            print("Single subset value of {subset} being interpreted as a high range")
            print(f"subset being set to a range of (0,{subset})\n")
            subset = [0, subset]

        # If the user has specified `subset` incorrectly, then let's return
        # an empty data and bucket
        if subset[1]-subset[0]<=0:
            print("The range in subset is either 0 or negative!")
            print(f"{subset[1]} - {subset[0]} = {subset[1] - subset[0]}")
            print("Returning an empty data and bucket dictionary!\n")
            return data,bucket

        # Make sure the user is not asking for something bigger than the file!
        nbuckets = data["_NUMBER_OF_BUCKETS_"]

        if subset[0] > nbuckets:
            print("Range for subset starts greater than number of buckets in file!")
            print(f"{subset[0]} > {nbuckets}")
            print(f"I'm not sure how to handle this so the file will not be opened.")
            print(f"Returning None,None")
            infile.close()
            return None,None

        if subset[1] > nbuckets:
            print("Range for subset is greater than number of buckets in file!")
            print(f"{subset[1]} > {nbuckets}")
            print(f"High range of subset will be set to {nbuckets}\n")
            subset[1] = nbuckets

        data["_NUMBER_OF_BUCKETS_"] = subset[1] - subset[0]
        nbuckets = data["_NUMBER_OF_BUCKETS_"]

        print("Will read in a subset of the file!")
        print(f"From bucket {subset[0]} (inclusive) through bucket {subset[1]-1} (inclusive)")
        print(f"Bucket {subset[1]} is not read in")
        print(f"Reading in {nbuckets} buckets\n")

    ############################################################################
    # Get the datasets and counters
    ############################################################################
    dc = infile["_MAP_DATASETS_TO_COUNTERS_"]
    for vals in dc:

        if verbose:
            print(f"Map datasets to counters: {vals}")

        # The decode is there because vals were stored as numpy.bytes
        counter = vals[1].decode()
        index = f"{counter}_INDEX"
        data["_MAP_DATASETS_TO_COUNTERS_"][vals[0].decode()] = counter
        data["_MAP_DATASETS_TO_INDEX_"][vals[0].decode()] = index
        data["_LIST_OF_COUNTERS_"].append(vals[1].decode())
        data["_LIST_OF_DATASETS_"].append(vals[0].decode())
        data["_LIST_OF_DATASETS_"].append(vals[1].decode())  # Get the counters as well

    # We may have added some counters and datasets multiple times.
    # So just to be sure, only keep the unique values
    data["_LIST_OF_COUNTERS_"] = np.unique(data["_LIST_OF_COUNTERS_"]).tolist()
    data["_LIST_OF_DATASETS_"] = np.unique(data["_LIST_OF_DATASETS_"]).tolist()
    ############################################################################

    ############################################################################
    # Pull out the SINGLETON datasets
    ############################################################################
    sg = infile["_SINGLETONSGROUPFORSTORAGE_"][0]  # This is a numpy array of strings
    decoded_string = sg[1].decode()

    vals = decoded_string.split("__:__")
    vals.remove("COUNTER")

    data["_SINGLETONS_GROUP_"] = vals
    ############################################################################

    ############################################################################
    # Get the list of datasets and groups
    ############################################################################
    all_datasets = data["_LIST_OF_DATASETS_"]

    if verbose:
        print(f"all_datasets: {all_datasets}")
    ############################################################################

    ############################################################################
    # Only keep select data from file, if we have specified desired_datasets
    ############################################################################
    if desired_datasets is not None:
        if type(desired_datasets) != list:
            desired_datasets = list(desired_datasets)

        # Count backwards because we'll be removing stuff as we go.
        i = len(all_datasets) - 1
        while i >= 0:
            entry = all_datasets[i]

            is_dropped = True
            # This is looking to see if the string is anywhere in the name
            # of the dataset
            for desdat in desired_datasets:
                if desdat in entry:
                    is_dropped = False
                    break

            if is_dropped == True:
                print(f"Not reading out {entry} from the file....")
                all_datasets.remove(entry)

            i -= 1

        if verbose:
            print(f"After only selecting certain datasets ----- ")
            print(f"all_datasets: {all_datasets}")
    ###########################################################################

    # We might need the counter for SINGLETONS so let's pull it out
    data["_SINGLETONS_GROUP_/COUNTER"] = infile["_SINGLETONS_GROUP_"]['COUNTER']

    if verbose == True:
        print("\nDatasets and counters:")
        print(data["_MAP_DATASETS_TO_COUNTERS_"])
        print("\nList of counters:")
        print(data["_LIST_OF_COUNTERS_"])
        print("\n_SINGLETONS_GROUP_/COUNTER:")
        print(data["_SINGLETONS_GROUP_/COUNTER"])
        print("\n")

    ############################################################################
    # Pull out the counters and build the indices
    ############################################################################
    print("Building the indices...\n")

    if verbose:
        print("data.keys()")
        print(data.keys())
        print("\n")

    # We will need to keep track of the indices in the entire file
    # This way, if the user specifies a subset of the data, we have the full 
    # indices already calculated
    full_file_indices = {}

    for counter_name in data["_LIST_OF_COUNTERS_"]:

        if verbose:
            print(f"counter name: ------------ {counter_name}\n")

        full_file_counters = infile[counter_name]
        full_file_index = calculate_index_from_counters(full_file_counters)

        if verbose:
            print(f"full file counters: {full_file_counters}\n")
            print(f"full file index: {full_file_index}\n")

        # If we passed in subset, grab that slice of the data from the file
        if subset is not None and subset[1] <= subset[0]:
            print("Will not be reading anything in!")
            print(f"High range of {subset[1]} is less than or equal to low range of {subset[0]}")
            print("Returning None,None...")
            return None,None

        elif subset is not None:
            # We tack on +1 to the high range of subset when we pull out the counters
            # and index because we want to get all of the entries for the last entry.
            data[counter_name] = infile[counter_name][subset[0] : subset[1]+1]
            index = full_file_index[subset[0] : subset[1]+1]
        else:
            data[counter_name] = infile[counter_name][:]
            index = full_file_index

        subset_index = index
        # If the file is *not* empty....
        # Just to make sure the "local" index of the data dictionary starts at 0
        if len(index)>0:
            subset_index = index - index[0]

        index_name = "%s_INDEX" % (counter_name)

        data[index_name] = subset_index
        full_file_indices[index_name] = index

    print("Built the indices!")

    if verbose:
        print("full_file_index: ")
        print(f"{full_file_indices}\n")

    # Loop over the all_datasets we want and pull out the data.
    for name in all_datasets:

        # If this is a counter, we're going to have to grab the indices
        # differently than for a "normal" dataset
        IS_COUNTER = True
        index_name = None
        if name not in data["_LIST_OF_COUNTERS_"]:
            index_name = data["_MAP_DATASETS_TO_INDEX_"][name]
            IS_COUNTER = False # We will use different indices for the counters

        if verbose == True:
            print(f"------ {name}")
            print(f"index_name: {index_name}\n")

        dataset = infile[name]

        if verbose:
            print(f"dataset type: {type(dataset)}")
        
        # This will ignore the groups
        if type(dataset) == h5.Dataset:
            dataset_name = name

            if subset is not None:
                if IS_COUNTER:
                    # If this is a counter, then the subset indices
                    # map on to the same locations for any counters
                    lo = subset[0]
                    hi = subset[1]
                else:
                    lo = full_file_indices[index_name][0]
                    hi = full_file_indices[index_name][-1]
                if verbose:
                    print(f"dataset name/lo/hi: {dataset_name},{lo},{hi}\n")
                data[dataset_name] = dataset[lo : hi]
            else:
                data[dataset_name] = dataset[:]

            bucket[dataset_name] = None  # This will be filled for individual bucket
            if verbose == True:
                print(dataset)

    infile.close()
    print("Data is read in and input file is closed.")

    return data, bucket


################################################################################

################################################################################
def calculate_index_from_counters(counters):

    index = np.add.accumulate(counters) - counters

    return index

################################################################################

################################################################################
def unpack(bucket, data, n=0):

    """ Fills the bucket dictionary with selected rows from the data dictionary.

    Args:

	**bucket** (dict): bucket dictionary to be filled

	**data** (dict): Data dictionary used to fill the bucket dictionary

    **n** (integer): 0 by default. Which entry should be pulled out of the data
                     dictionary and inserted into the bucket dictionary.

    """

    keys = bucket.keys()

    for key in keys:

        # if "num" in key:
        # IS THERE A WAY THAT THIS COULD BE FASTER?
        # print(data['_LIST_OF_COUNTERS_'],key)
        if key in data["_LIST_OF_COUNTERS_"] or key in data["_SINGLETONS_GROUP_"]:
            bucket[key] = data[key][n]

        elif "INDEX" not in key:  # and 'Jets' in key:
            indexkey = data["_MAP_DATASETS_TO_INDEX_"][key]
            numkey = data["_MAP_DATASETS_TO_COUNTERS_"][key]

            if len(data[indexkey]) > 0:
                index = data[indexkey][n]

            if len(data[numkey]) > 0:
                nobjs = data[numkey][n]
                bucket[key] = data[key][index : index + nobjs]


################################################################################
def get_nbuckets_in_file(filename):

    """ Get the number of buckets in the file.

    """

    #f = h5.File(filename, "r+")
    #a = f.attrs

    if type(filename) is not str:
        print(f"Expecting a string for the filename!")
        return None

    with h5.File(filename, "r+") as f:
        a = f.attrs

        if a.__contains__("_NUMBER_OF_BUCKETS_"):
            _NUMBER_OF_BUCKETS_ = a.get("_NUMBER_OF_BUCKETS_")
            f.close()
            return _NUMBER_OF_BUCKETS_
        else:
            print('\nFile does not contain the attribute, "_NUMBER_OF_BUCKETS_"\n')
            f.close()
            return None

################################################################################
def get_nbuckets_in_data(data):

    """ Get the number of buckets in the data dictionary.
        
        This is useful in case you've only pulled out subsets of the data

    """

    if type(data) is not dict:
        print(f"{data} is not a dictionary!\n")
        return None

    if "_NUMBER_OF_BUCKETS_" in list(data.keys()):
        _NUMBER_OF_BUCKETS_ = data["_NUMBER_OF_BUCKETS_"]
        return _NUMBER_OF_BUCKETS_
    else:
        print('\ndata dictionary does not contain the key, "_NUMBER_OF_BUCKETS_"\n')
        return None


################################################################################
def get_file_metadata(filename):

    """ Get the file metadata and return it as a dictionary

    """

    f = h5.File(filename, "r+")

    a = f.attrs

    if len(a) < 1:
        print(f"No metadata in file {filename}!")
        print(f"File has no attributes.\n")
        f.close()
        return None

    metadata = {}

    for key in a.keys():
        metadata[key] = a[key]

    f.close()

    return metadata


################################################################################

################################################################################
def print_file_metadata(filename):

    """ Pretty print the file metadata 

    """

    metadata = get_file_metadata(filename)

    if metadata is None:
        return None

    output = ""

    keys = list(metadata.keys())

    first_keys_to_print = ["date", "_NUMBER_OF_BUCKETS_"]

    keys_already_printed = []

    # Print the basics first
    for key in first_keys_to_print:
        if key in keys:
            val = metadata[key]
            output += f"{key:<20s} : {val}\n"
            keys_already_printed.append(key)

    # Print the versions next
    for key in keys:
        if key in keys_already_printed:
            continue

        if key.find("version") >= 0:
            val = metadata[key]
            output += f"{key:<20s} : {val}\n"
            keys_already_printed.append(key)

    # Print the read of the metadata
    for key in keys:
        if key in keys_already_printed:
            continue

        val = metadata[key]
        output += f"{key:<20s} : {val}\n"
        keys_already_printed.append(key)

    print(output)

    return output


################################################################################



import cProfile

import numpy as np
import sys
import hepfile

# For development
#sys.path.append('../src/hepfile')
#import write as hepfile

def main():
    data = hepfile.initialize()

    hepfile.create_group(data,'jet',counter='njet')
    hepfile.create_dataset(data,['e','px','py','pz'],group='jet',dtype=float)
    hepfile.create_dataset(data,['algorithm'],group='jet',dtype=int)
    hepfile.create_dataset(data,['words'],group='jet',dtype=str)

    hepfile.create_group(data,'muons',counter='nmuon')
    hepfile.create_dataset(data,['e','px','py','pz'],group='muons',dtype=float)

    hepfile.create_dataset(data,['METpx','METpy'],dtype=float)

    event = hepfile.create_single_bucket(data)

    rando_words = ["hi", "bye", "ciao", "aloha"]

    print("Created the datasets and groups...")

#'''
    for i in range(0,100000):

        #hepfile.clear_event(event)

        njet = 17
        event['jet/njet'] = njet

        for n in range(njet):
            event['jet/e'].append(np.random.random())
            event['jet/px'].append(np.random.random())
            event['jet/py'].append(np.random.random())
            event['jet/pz'].append(np.random.random())

            event['jet/algorithm'].append(np.random.randint(-1,1))

            event['jet/words'].append(np.random.choice(rando_words))

        event['METpx'] = np.random.random()
        event['METpy'] = np.random.random()

        #hepfile.pack(data,event,EMPTY_OUT_BUCKET=False)
        return_value = hepfile.pack(data,event,STRICT_CHECKING=True)
        if return_value != 0:
            exit()

    print("Writing the file...")
#hdfile = hepfile.write_to_file('output.hdf5',data)
    hdfile = hepfile.write_to_file('output.hdf5',data,comp_type='gzip',comp_opts=9,verbose=True)
#'''

if __name__ == "__main__":
    profiler = cProfile.Profile()
    profiler.enable()
    main()
    profiler.disable()
    profiler.dump_stats("example.prof")



import numpy as np
import h5py as h5
import hepfile

import matplotlib.pylab as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import colors

###################################################
# Fill a matrix with a random amount of points.
# Store many of these matrices using h5hep.
###################################################

# Number of matrices
nmat = 4

# Assign matrix dimensions
x_size = 25
y_size = 25
x_size = int(x_size)
y_size = int(y_size)

# Subplot size for later
px_size = 2
py_size = 2

# Initialize the data dictionary for a single matrix
data = hepfile.initialize()
    
#Create groups
hepfile.create_group(data, "pixels", counter="npixels")

# Create datasets within the groups
hepfile.create_dataset(data, ["x","y","value"], group="pixels")

# Create an event from the data container
event = hepfile.create_single_event(data)

for i in range(0,nmat):
    # Random selection of number of pixels in the matrix
    r = np.random.randint(0,x_size*y_size)

    # For each point, randomly select an x and y position
    # Repeats are ok in this example
    for i in range(0,r):
        event['pixels/x'].append(np.random.randint(1,x_size+1))
        event['pixels/y'].append(np.random.randint(1,y_size+1))
        event['pixels/value'].append(np.random.randint(1,70))

    # Counters are the same here because there is always one data point per point
    # They could be different if there was more data collected for each point
    event['pixels/npixels'] = r
    
    # Add the event to the matrix dictionary
    hepfile.pack(data,event)

    # Clear the event matrix for the next iteration
    hepfile.clear_event(event)

###############
# Write to file
###############
filename = 'matrix_example_output.hdf5'
hdfile = hepfile.write_to_file(filename,data,comp_type='gzip',comp_opts=9)

################
# Read from file
################
data,event = hepfile.load(filename)
nevents = data['nentries']
print(data)
print(f"nevents: {nevents}")
#nevents = data['nevents']
fig = plt.figure(figsize=(10,10))
for i in range(0,nevents):

    hepfile.unpack(event,data,n=i)

    print(event.keys())

    npixels = event['pixels/npixels']
    x = event['pixels/x']
    y = event['pixels/y']
    d = event['pixels/value']

    grid = np.zeros((x_size,y_size))

    for j in range(npixels):
        grid[x[j]-1,y[j]-1] = d[j]

    # create discrete colormap
    #cmap = colors.ListedColormap(['red', 'blue'])
    cmap = plt.get_cmap('plasma')

    ax = fig.add_subplot(px_size,py_size,i+1)
    ax.imshow(grid, cmap=cmap)

    # draw gridlines
    ax.grid(which='major', axis='both', linestyle='-', color='k', linewidth=2)
    ax.set_xticks(np.arange(-.5, x_size, 1));
    ax.set_yticks(np.arange(-.5, y_size, 1));

    ax.set_xticklabels(range(0,x_size))
    ax.set_yticklabels(range(0,y_size))
    plt.tight_layout()

plt.show()















import numpy as np
import hepfile
import h5py as h5

import time

import sys
sys.path.append('./scripts')
#from write_h5hep_file_for_unit_tests import write_h5hep_file_for_unit_tests

# sys.path.append('../src/hepfile')
#import write as hepfile
#import read as read

#from write_file_for_unit_tests import write_file_for_unit_tests


############################################################################
def isEmpty(dictionary):
    test = True
    print(dictionary.keys())
    for key in dictionary.keys():
        print(key)
        print(dictionary[key])
        print(type(dictionary[key]))
        if dictionary[key] is None:
            test = True
        elif type(dictionary[key]) == list or type(dictionary[key]) == np.ndarray:
            if len(dictionary[key]) > 0:
                test = False

    return test
################################################################################

################################################################################


def test_initialize():

    test_data = hepfile.initialize()

    print(test_data)

    assert isinstance(test_data, dict)
    assert test_data['_GROUPS_']['_SINGLETONS_GROUP_'] == ['COUNTER']
    assert test_data['_MAP_DATASETS_TO_COUNTERS_']['_SINGLETONS_GROUP_'] == '_SINGLETONS_GROUP_/COUNTER'
    assert test_data['_LIST_OF_COUNTERS_'] == ['_SINGLETONS_GROUP_/COUNTER']
    assert test_data['_SINGLETONS_GROUP_/COUNTER'] == []
################################################################################


################################################################################
def test_clear_bucket():

    # This assumes you run nosetests from the h5hep directory and not
    # the tests directory.
    filename = "FOR_TESTS.hdf5"
    desired_datasets = ['jet', 'muon']
    subset = 1000

    data, bucket = hepfile.load(filename, False, desired_datasets, subset)

    hepfile.clear_bucket(bucket)

    assert isEmpty(bucket) == True
################################################################################


################################################################################
def test_create_single_bucket():

    data = hepfile.initialize()

    hepfile.create_group(data, 'jet', counter='njet')
    hepfile.create_dataset(
        data, ['e', 'px', 'py', 'pz'], group='jet', dtype=float)

    hepfile.create_group(data, 'muons', counter='nmuon')
    hepfile.create_dataset(
        data, ['e', 'px', 'py', 'pz'], group='muons', dtype=float)

    test_bucket = hepfile.create_single_bucket(data)

    assert isEmpty(test_bucket) == False
    assert isinstance(test_bucket, dict)
################################################################################

################################################################################


def test_create_group():

    data = hepfile.initialize()
    hepfile.create_group(data, 'jet', counter='njet')

    assert isEmpty(data['_GROUPS_']) == False
    assert 'jet/njet' in data.keys()

    hepfile.create_group(data,"test/slash",counter='ntest/slash')

    assert 'test-slash' in data['_GROUPS_']
    assert 'test-slash/ntest-slash' in data.keys()

################################################################################

################################################################################


def test_pack():

    data = hepfile.initialize()
    hepfile.create_group(data, 'obj', counter='nobj')
    hepfile.create_dataset(data, ['myfloat'], group='obj', dtype=float)
    hepfile.create_dataset(data, ['myint'], group='obj', dtype=int)
    hepfile.create_dataset(data, ['mystr'], group='obj', dtype=str)

    bucket = hepfile.create_single_bucket(data)

    # Normal packing test

    for i in range(5):
        bucket['obj/myfloat'].append(2.0)
        bucket['obj/myint'].append(2)
        bucket['obj/mystr'].append('two')
    bucket['obj/nobj'] = 5

    test = hepfile.pack(data, bucket)
    assert test == 0
    assert len(data['obj/myfloat']) == 5
    assert len(data['obj/myint']) == 5
    assert len(data['obj/mystr']) == 5
    assert data['obj/nobj'][0] == 5

    assert len(bucket['obj/myfloat']) == 0
    assert len(bucket['obj/myint']) == 0
    assert len(bucket['obj/mystr']) == 0
    assert bucket['obj/nobj'] is 0

    # AUTO_SET_COUNTER = False
    bucket['obj/myfloat'].append(2.0)
    bucket['obj/myint'].append(2)
    bucket['obj/mystr'].append('two')

    # Is the mistake propagated?
    bucket['obj/nobj'] = 2

    hepfile.pack(data, bucket, AUTO_SET_COUNTER=False)
    assert data['obj/nobj'][1] == 2

    # Fix mistake
    data['obj/nobj'][1] = 2

    # STRICT_CHECKING = True
    bucket['obj/myfloat'].append(2.0)
    bucket['obj/myint'].append(2)
    # 1 != 0, strict checking should fail.

    test = hepfile.pack(data, bucket, STRICT_CHECKING=True)

    # Was the mistake caught?
    assert test == -1
    # Was nothing packed?
    assert len(data['obj/myint']) == 6
    # Is bucket not cleared?
    assert isEmpty(bucket) == False

    # EMPTY_OUT_BUCKET = False

    bucket['obj/mystr'].append('two')

    hepfile.pack(data, bucket, EMPTY_OUT_BUCKET=False)

    assert isEmpty(bucket) == False

    #assert type(data['obj/mystr'][0]) is str


################################################################################


################################################################################
def test_create_dataset():

    data = hepfile.initialize()
    hepfile.create_group(data, 'jet', counter='njet')
    hepfile.create_dataset(
        data, ['e', 'px', 'py', 'pz'], group='jet', dtype=float)
    hepfile.create_dataset(data, 'METpx', dtype=int)

    assert isEmpty(data['_GROUPS_']) == False
    assert 'jet/njet' in data.keys()
    assert 'jet/e' in data.keys()
    assert 'jet/px' in data.keys()
    assert 'jet/e' in data['_MAP_DATASETS_TO_COUNTERS_'].keys()
    assert data['_MAP_DATASETS_TO_COUNTERS_']['jet/e'] == 'jet/njet'
    assert data["_MAP_DATASETS_TO_DATA_TYPES_"]['jet/e'] == float

    assert 'METpx' in data["_GROUPS_"]["_SINGLETONS_GROUP_"]
    assert data["_MAP_DATASETS_TO_COUNTERS_"]['METpx'] == "_SINGLETONS_GROUP_/COUNTER"
    assert data["_MAP_DATASETS_TO_DATA_TYPES_"]['METpx'] == int
################################################################################


################################################################################
def test_write_file_metadata():

    filename = "FOR_TESTS.hdf5"
    file = h5.File(filename, "r")

    # Check default attribute existence
    assert 'date' in file.attrs.keys()
    assert 'hepfile_version' in file.attrs.keys()
    assert 'h5py_version' in file.attrs.keys()
    assert 'numpy_version' in file.attrs.keys()
    assert 'python_version' in file.attrs.keys()

    # Check default attributes are strings
    assert isinstance(file.attrs['date'], str)
    assert isinstance(file.attrs['hepfile_version'], str)
    assert isinstance(file.attrs['h5py_version'], str)
    assert isinstance(file.attrs['numpy_version'], str)
    assert isinstance(file.attrs['python_version'], str)

    file.close()

    # Adding a new attribute
    hepfile.write_file_metadata(filename, {'author': 'John Doe'})
    file = h5.File(filename, "r")

    assert 'author' in file.attrs.keys()
    assert file.attrs['author'] == 'John Doe'

    file.close()
################################################################################



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['hepfile']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['numpy >=1.13.3', 'h5py', 'awkward >= 1.4', 'pre-commit']

extras_require = \
{":python_version<'3.5'": ['typing'],
 'awkward': ['awkward >= 1.4'],
 'dev': ['pytest >=6'],
 'docs': ['Sphinx >=3.0.0',
          'myst_parser>=0.13',
          'sphinx-book-theme>=0.0.33',
          'sphinx_copybutton'],
 'test': ['pytest >=6']}

setup(name='hepfile',
      version='0.1.3',
      description='A file description modeled after the ROOT analysis toolkit and common use-cases in High Energy Physics. Implemented in HDF5, hepfile stands for Heterogeneous Entries in Parallel-file. ',
      author='Matt Bellis',
      author_email='matthew.bellis@gmail.com',
      url='https://github.com/mattbellis/hepfile',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )



import h5py as h5
import numpy as np
import matplotlib.pylab as plt
import time

import hepfile

import sys

filename = sys.argv[1]

energies = []

for nfiles in range(0,30):

    data,event = hepfile.load(filename)

    # Print out what has been read in from the files.
    '''
    for key in event.keys():
        print(key)
    '''

    nentries = data['nentries']
    print(f"nentries: {nentries}")

    energies += data['jet/e'].tolist()

    '''
    for i in range(0,nentries):

        if i%10000==0:
            print(i)

        hepfile.unpack(event,data,n=i)

        energy = event['jet/e']

        energies += energy.tolist()
    '''

    del data
    del event


print(len(energies))

plt.figure()
plt.hist(energies,bins=100,range=(0,500))

#plt.show()
